//Microsoft Developer Studio generated resource script.
//
#include "resource.h"

1 24 "uninstall.exe.manifest"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//


/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (U.S.) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENU)
#ifdef _WIN32
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
#pragma code_page(1252)
#endif //_WIN32

#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE DISCARDABLE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

3 TEXTINCLUDE DISCARDABLE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


#ifndef _MAC
/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION 1,0,0,2
 PRODUCTVERSION 1,0,0,2
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x40004L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904b0"
        BEGIN
            VALUE "Comments", "\0"
            VALUE "CompanyName", "mozilla.org\0"
            VALUE "FileDescription", "uninstall\0"
            VALUE "FileVersion", "1, 0, 0, 2\0"
            VALUE "InternalName", "uninstall\0"
            VALUE "LegalCopyright", "Copyright © 2001\0"
            VALUE "LegalTrademarks", "\0"
            VALUE "OriginalFilename", "uninstall.exe\0"
            VALUE "PrivateBuild", "\0"
            VALUE "ProductName", "Mozilla uninstaller\0"
            VALUE "ProductVersion", "1, 0, 0, 2\0"
            VALUE "SpecialBuild", "\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x409, 1200
    END
END

#endif    // !_MAC


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_UNINSTALL           ICON    DISCARDABLE     "uninstall.ico"

/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

DLG_UNINSTALL DIALOG DISCARDABLE  51, 56, 225, 79
STYLE WS_MINIMIZEBOX | WS_POPUP | WS_VISIBLE | WS_CAPTION | WS_SYSMENU
CLASS "MozillaSetupDlg"
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "",IDWIZNEXT,54,55,53,14
    PUSHBUTTON      "",IDCANCEL,118,55,53,14
    LTEXT           "",IDC_MESSAGE0,17,11,191,32,NOT WS_GROUP
END

DLG_MESSAGE DIALOG DISCARDABLE  0, 0, 236, 34
STYLE DS_MODALFRAME | WS_POPUP | WS_VISIBLE
FONT 8, "MS Sans Serif"
BEGIN
    CTEXT           "",IDC_MESSAGE,0,0,236,34,SS_CENTERIMAGE
END

DLG_MESSAGE_CHK DIALOG DISCARDABLE  0, 0, 200, 66
STYLE DS_SETFONT | DS_MODALFRAME | DS_FIXEDSYS | WS_POPUP | WS_CAPTION | WS_SYSMENU
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "",IDOK,75,45,53,14
    LTEXT           "",IDC_MESSAGE,11,8,191,8
    CONTROL         "",IDC_CHECKBOX,"Button",BS_AUTOCHECKBOX|WS_TABSTOP,15,25,191,10
END

DLG_WHAT_TO_DO DIALOG DISCARDABLE  51, 56, 333, 140
STYLE DS_MODALFRAME | WS_MINIMIZEBOX | WS_POPUP | WS_VISIBLE | WS_CAPTION | 
    WS_SYSMENU
FONT 8, "MS Sans Serif"
BEGIN
    DEFPUSHBUTTON   "",ID_NO,40,108,53,14
    PUSHBUTTON      "",ID_NO_TO_ALL,107,108,53,14
    PUSHBUTTON      "",ID_YES,174,108,53,14
    PUSHBUTTON      "",ID_YES_TO_ALL,241,108,53,14
    LTEXT           "",IDC_MESSAGE0,16,13,301,33,NOT WS_GROUP
    LTEXT           "",IDC_MESSAGE1,16,50,301,19,NOT WS_GROUP
    RTEXT           "",IDC_STATIC,18,81,41,8
    LTEXT           "Static",IDC_STATIC_SHARED_FILENAME,66,79,251,14,
                    SS_CENTERIMAGE | SS_SUNKEN
END


/////////////////////////////////////////////////////////////////////////////
//
// String Table
//

STRINGTABLE DISCARDABLE 
BEGIN
    IDS_ERROR_DLL_LOAD      "Could not load %s"
    IDS_ERROR_STRING_LOAD   "Could not load string resource ID %d"
    IDS_ERROR_STRING_NULL   "Null pointer encountered."
    IDS_ERROR_GLOBALALLOC   "Memory allocation error."
END

#endif    // English (U.S.) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

